@@ -20,18 +20,20 @@ come with the distribution.
 
 =cut
 
-$VERSION = '0.98';
+$VERSION = '0.99';
 use strict;
 use B;
 use Sepia::Debug;               # THIS TURNS ON DEBUGGING INFORMATION!
 use Cwd 'abs_path';
 use Scalar::Util 'looks_like_number';
 use Text::Abbrev;
+use File::Find;
+use Storable qw(store retrieve);
 
 use vars qw($PS1 %REPL %RK %REPL_DOC %REPL_SHORT %PRINTER
             @REPL_RESULT @res
             $REPL_LEVEL $PACKAGE $WANTARRAY $PRINTER $STRICT $PRINT_PRETTY
-            $ISEVAL);
+            $ISEVAL $LAST_INPUT);
 
 sub repl_strict
 {
@@ -250,6 +252,21 @@ sub all_completions
     map { s/^:://; $_ } _completions('::', @parts);
 }
 
+# Filter exact matches so that e.g. "A::x" completes to "A::xx" when
+# both "Ay::xx" and "A::xx" exist.
+sub filter_exact_prefix
+{
+    my @parts = split /:+/, shift, -1;
+    my @res = @_;
+    my @tmp;
+    my $pre = shift @parts;
+    while (@parts && (@tmp = grep /^\Q$pre\E(?:::|$)/, @res)) {
+        @res = @tmp;
+        $pre .= '::'.shift @parts;
+    }
+    @res;
+}
+
 sub lexical_completions
 {
     eval { require PadWalker; import PadWalker 'peek_sub' };
@@ -294,7 +311,7 @@ sub completions
             $type ? filter_typed $type : filter_untyped
         } all_abbrev_completions $str;
     }
-    @ret = map { s/^:://; "$t$_" } @ret;
+    @ret = map { s/^:://; "$t$_" } filter_exact_prefix $str, @ret;
 #     ## XXX: Control characters, $", and $1, etc. confuse Emacs, so
 #     ## remove them.
     grep {
@@ -738,13 +755,62 @@ sub flow
     $_
 }
 
+sub load
+{
+    my $a = shift;
+    no strict;
+    for (@$a) {
+        *{$_->[0]} = $_->[1];
+    }
+}
+
+my %BADVARS;
+undef @BADVARS{qw(%INC @INC %SIG @ISA %ENV @ARGV)};
+
+# magic variables
+sub saveable
+{
+    local $_ = shift;
+    return !/^.[^c-zA-Z]$/ # single-letter stuff (match vars, $_, etc.)
+        && !/^.[\0-\060]/         # magic weirdness.
+        && !/^._</        # debugger info
+        && !exists $BADVARS{$_}; # others.
+}
+
+sub save
+{
+    my ($re) = @_;
+    my @save;
+    $re = qr/(?:^|::)$re/;
+    no strict;                  # no kidding...
+    my_walksymtable {
+        return if /::$/
+            || $stash =~ /^(?:::)?(?:warnings|Config|strict|B)\b/;
+        if (/$re/) {
+            my $name = "$stash$_";
+            if (defined ${$name} and saveable '$'.$_) {
+                push @save, [$name, \$$name];
+            }
+            if (defined *{$name}{HASH} and saveable '%'.$_) {
+                push @save, [$name, \%{$name}];
+            }
+            if (defined *{$name}{ARRAY} and saveable '@'.$_) {
+                push @save, [$name, \@{$name}];
+            }
+        }
+    } '::';
+    print STDERR "$_->[0] " for @save;
+    print STDERR "\n";
+    \@save;
+}
+
 =head2 C<define_shortcut $name, $sub [, $doc [, $shortdoc]]>
 
 Define $name as a shortcut for function $sub.
 
 =cut
 
-    sub define_shortcut
+sub define_shortcut
 {
     my ($name, $doc, $short, $fn);
     if (@_ == 2) {
@@ -787,7 +853,7 @@ sub define_shortcuts
         'strict [0|1]', 'Turn \'use strict\' mode on or off';
     define_shortcut 'quit', \&Sepia::repl_quit,
         'Quit the REPL';
-    define_shortcut 'reload', \&Sepia::repl_reload,
+    define_shortcut 'restart', \&Sepia::repl_restart,
         'Reload Sepia.pm and relaunch the REPL.';
     define_shortcut 'shell', \&Sepia::repl_shell,
         'shell CMD ...', 'Run CMD in the shell';
@@ -801,6 +867,16 @@ sub define_shortcuts
             'Define NAME as a shortcut executing BODY';
     define_shortcut undef => \&Sepia::repl_undef,
         'undef NAME', 'Undefine shortcut NAME';
+    define_shortcut test => \&Sepia::repl_test,
+        'test FILE...', 'Run tests interactively.';
+    define_shortcut load => \&Sepia::repl_load,
+        'load [FILE]', 'Load state from FILE.';
+    define_shortcut save => \&Sepia::repl_save,
+        'save [PATTERN [FILE]]', 'Save variables matching PATTERN to FILE.';
+    define_shortcut reload => \&Sepia::repl_reload,
+        'reload [MODULE | /RE/]', 'Reload MODULE, or all modules matching RE.';
+    define_shortcut freload => \&Sepia::repl_full_reload,
+        'freload MODULE', 'Reload MODULE and all its dependencies.';
 }
 
 sub repl_help
@@ -1025,11 +1101,11 @@ sub repl_quit
     last repl;
 }
 
-sub repl_reload
+sub repl_restart
 {
     do $INC{'Sepia.pm'};
     if ($@) {
-        print "Reload failed:\n$@\n";
+        print "Restart failed:\n$@\n";
     } else {
         $REPL_LEVEL = 0;        # ok?
         goto &Sepia::repl;
@@ -1069,6 +1145,97 @@ sub repl_eval
     }
 }
 
+sub repl_test
+{
+    my ($buf) = @_;
+    my @files;
+    if ($buf =~ /\S/) {
+        $buf =~ s/^\s+//;
+        $buf =~ s/\s+$//;
+        if (-f $buf) {
+            push @files, $buf;
+        } elsif (-f "t/$buf") {
+            push @files, $buf;
+        } else {
+            return;
+        }
+    } else {
+        find({ no_chdir => 1,
+               wanted => sub {
+                   push @files, $_ if /\.t$/;
+            }}, Cwd::getcwd() =~ /t\/?$/ ? '.' : './t');
+    }
+}
+
+sub repl_load
+{
+    my ($file) = split ' ', shift;
+    $file ||= "$ENV{HOME}/.sepia-save";
+    load(retrieve $file);
+}
+
+sub repl_save
+{
+    my ($re, $file) = split ' ', shift;
+    $re ||= '.';
+    $file ||= "$ENV{HOME}/.sepia-save";
+    store save($re), $file;
+}
+
+sub full_reload
+{
+    (my $name = shift) =~ s!::!/!g;
+    $name .= '.pm';
+    print STDERR "full reload $name\n";
+    my %save_inc = %INC;
+    local %INC;
+    require $name;
+    my @ret = keys %INC;
+    while (my ($k, $v) = each %save_inc) {
+        $INC{$k} ||= $v;
+    }
+    @ret;
+}
+
+sub repl_full_reload
+{
+    chomp (my $pat = shift);
+    my @x = full_reload $pat;
+    print "Reloaded: @x\n";
+}
+
+sub repl_reload
+{
+    chomp (my $pat = shift);
+    if ($pat =~ /^\/(.*)\/?$/) {
+        $pat = $1;
+        $pat =~ s#::#/#g;
+        $pat = qr/$pat/;
+        my @rel;
+        for (keys %INC) {
+            next unless /$pat/;
+            if (!do $_) {
+                print "$_: $@\n";
+            }
+            s#/#::#g;
+            s/\.pm$//;
+            push @rel, $_;
+        }
+    } else {
+        my $mod = $pat;
+        $pat =~ s#::#/#g;
+        $pat .= '.pm';
+        if (exists $INC{$pat}) {
+            delete $INC{$pat};
+            eval 'require $mod';
+            import $mod if $@;
+            print "Reloaded $mod.\n"
+        } else {
+            print "$mod not loaded.\n"
+        }
+    }
+}
+
 ## Collects warnings for REPL
 my @warn;
 
@@ -1202,6 +1369,14 @@ sub repl
             unless ($SIG{__WARN__}) {
                 $SIG{__WARN__} = 'Sepia::sig_warn';
             }
+            if (!$ISEVAL) {
+                if ($buf eq '') {
+                    # repeat last interactive command
+                    $buf = $LAST_INPUT;
+                } else {
+                    $LAST_INPUT = $buf;
+                }
+            }
             if ($buf =~ /^,(\S+)\s*(.*)/s) {
                 ## Inspector shortcuts
                 my $short = $1;
@@ -1376,6 +1551,22 @@ sub html_package_list
     $file ? 1 : $out;
 }
 
+sub apropos_module
+{
+    my $re = qr/$_[0]/;
+    my $inst = inst();
+    my %ret;
+    for (package_list) {
+        undef $ret{$_} if /$re/;
+    }
+    undef $ret{$_} for map {
+        s/.*man.\///; s|/|::|g; s/\.\d(?:pm)?$//; $_
+    } grep {
+        /\.\d(?:pm)?$/ && !/man1/ && !/usr\/bin/ && /$re/
+    } $inst->files('Perl');
+    sort keys %ret;
+}
+
 1;
 __END__
 
