@@ -1,9 +1,12 @@
 ;;; Sepia -- Simple Emacs-Perl InterAction: ugly, yet effective.
-;;; (a.k.a. Septik -- Sean's Emacs-Perl Total Integration Kludge.)
+;;  (a.k.a. Septik -- Sean's Emacs-Perl Total Integration Kludge.)
 
-;; Copyright (C) 2004-2008 Sean O'Rourke.  All rights reserved, some
-;; wrongs reversed.  This code is distributed under the same terms as
-;; Perl itself.
+;; Author: Sean O'Rourke <seano@cpan.org>
+;; Keywords: Perl, languages
+
+;; Copyright (C) 2004-2009 Sean O'Rourke.  All rights reserved, some
+;;   wrongs reversed.  This code is distributed under the same terms
+;;   as Perl itself.
 
 ;;; Commentary:
 
@@ -44,7 +47,7 @@
 Useful values include `sepia-w3m-view-pod' and `sepia-perldoc-buffer'.")
 
 (defvar sepia-module-list-function
-  (if (featurep 'w3m) 'w3m-find-file 'browse-url-of-buffer)
+  (if (featurep 'w3m) 'w3m-find-file 'browse-url-of-file)
 "* Function to view a list of installed modules.
 
 Useful values include `w3m-find-file' and `browse-url-of-buffer'.")
@@ -732,6 +735,28 @@ The prefix argument is the same as for `end-of-defun'."
       (forward-sexp))
     (point)))
 
+(defun sepia-rename-lexical (old new &optional prompt)
+  "Replace lexical variable OLD with NEW in the current function.
+
+With prefix argument, query for each replacement.  It is an error
+to call this outside a function."
+  (interactive
+   (let ((old (sepia-thing-at-point 'symbol)))
+     (list (read-string "Old name: " old nil old)
+           (read-string "New name: ")
+           current-prefix-arg)))
+   (message "(%s %s)" old new)
+   (unless (sepia-inside-defun)
+     (error "Can't rename %s outside a defun." old))
+   (setq old (concat "\\([$%@]\\)\\_<" (regexp-quote old) "\\_>")
+         new 
+        (concat "\\1" new))
+   (let ((bod (sepia-beginning-of-defun))
+         (eod (sepia-end-of-defun)))
+     (if prompt
+       (query-replace-regexp old new nil bod eod)
+       (replace-regexp old new nil bod eod))))
+
 (defun sepia-defun-around-point (&optional where)
   "Return the text of function around point."
   (unless where
@@ -1144,6 +1169,7 @@ Does not require loading.")
                  (mod-file "Find the file defining a package.")
                  (apropos "Find subnames matching RE.")
                  (lexicals "Find lexicals for a sub.")
+                 (apropos-module "Find installed modules matching RE.")
                  ))
       (apply #'define-xref-function "Sepia" x))
 
@@ -1264,7 +1290,7 @@ prefix arg, replace the region with the result."
 
 \(i.e. evaluate an expression on each line of region).  With
 prefix arg, replace the region with the result."
-  (interactive "MExpression:\nr\nP")
+  (interactive "MExpression: \nr\nP")
   (sepia-perlize-region-internal
    "do { my $ret='';my $region = "
    (concat "; for (split /(?<=\\n)/, $region, -1) { $ret .= do { " expr
@@ -1275,7 +1301,7 @@ prefix arg, replace the region with the result."
   "Evaluate a Perl expression on the region as a whole.
 
 With prefix arg, replace the region with the result."
-  (interactive "MExpression:\nr\nP")
+  (interactive "MExpression: \nr\nP")
   (sepia-perlize-region-internal
    "do { local $_ = " (concat "; do { " expr ";}; $_ }") beg end replace-p))
 
@@ -1303,6 +1329,15 @@ With prefix arg, replace the region with the result."
 	;; (car (xref-file-modules file))
 	(sepia-buffer-package))))
 
+;;;###autoload
+(defun sepia-apropos-module (name)
+  (interactive "MList modules matching regexp: ")
+  (let ((res (xref-apropos-module name)))
+    (if res
+        (with-output-to-temp-buffer "*Modules*"
+          (display-completion-list res))
+      (message "No modules matching %s." name))))
+
 ;;;###autoload
 (defun sepia-eval-defun ()
   "Re-evaluate the current function and rebuild its Xrefs."
@@ -1502,7 +1537,7 @@ calling `cperl-describe-perl-symbol'."
                   (if (and
                        (let ((bol (save-excursion (beginning-of-line)
                                                   (point))))
-                         (looking-back " *\\(?:use\\|require\\|package\\) +[^ ]+" bol))
+                         (looking-back " *\\(?:use\\|require\\|package\\|no\\)\\s +[^ ]*" bol))
                        (sepia-looks-like-module obj))
                       (sepia-core-version obj)
                       ""))))
